"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

"{ Package: 'stx:libbasic2' }"

Time subclass:#TimeDuration
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Magnitude-Time'
!

!TimeDuration class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1989 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
"
! !

!TimeDuration class methodsFor:'instance creation'!

fromSeconds:secondsInterval
    "redefined to disable wrapping at 24hours."

    ^ self new setSeconds:secondsInterval

    "
     Time hours:2 minutes:33 seconds:0 
     Time hours:100 minutes:33 seconds:0
     TimeDuration hours:2 minutes:33 seconds:0 millis:123  
     TimeDuration hours:100 minutes:33 seconds:0 millis:123
    "
!

hours:h minutes:m seconds:s millis:millis
    "return an instance of Time representing the given time.
     See also Time now / Date today / AbsoluteTime now."

    ^ self basicNew setHours:h minutes:m seconds:s millis:millis

    "
     TimeDuration hours:2 minutes:33 seconds:0 millis:123  
     TimeDuration hours:100 minutes:33 seconds:0 millis:123  
    "
! !

!TimeDuration class methodsFor:'format strings'!

formatString12us
    "return the format string used to format US times (and other areas)"

    ^ '%u:%m:%s.%i %a'
!

formatString24
    "return the format string used to format european times (and other areas)"

    ^ '%h:%m:%s.%i'
! !

!TimeDuration methodsFor:'accessing'!

days
    ^ timeEncoding // 1000 // 3600 // 24
!

milliseconds
    ^ timeEncoding \\ 1000
! !

!TimeDuration methodsFor:'converting'!

asTime
    "return a Time object from the receiver - thats the receiver."

    ^ Time hours:(self hours) minutes:(self minutes) seconds:(self seconds)
! !

!TimeDuration methodsFor:'printing'!

addPrintBindingsTo:aDictionary language:languageOrNil
    "private print support: add bindings for printing to aDictionary.
     languageOrNil can only be #en or nil for the current language."

    |hoursInDay s|

    super addPrintBindingsTo:aDictionary language:languageOrNil.
    aDictionary at:$d put:self days.

    hoursInDay := self hours \\ 24.
    aDictionary at:#Hd put:(s := hoursInDay printString).
    aDictionary at:#hd put:(s leftPaddedTo:2 with:$0).
!

printOn:aStream
    "append a user printed representation of the receiver to aStream.
     The format is suitable for a human - not meant to be read back."

    |fmt hr|

    (hr := self hours) >= 24 ifTrue:[
        fmt := '%dd %(Hd)h %Mm %S'
    ] ifFalse:[
        hr > 0 ifTrue:[
            fmt := '%Hh %Mm %S'
        ] ifFalse:[
            self minutes > 0 ifTrue:[
                fmt := '%Mm %S'
            ] ifFalse:[
                fmt := '%S'
            ].
        ].
    ].
    (self milliseconds ~~ 0) ifTrue:[
        fmt := fmt , '.%is'
    ] ifFalse:[
        fmt := fmt , 's'
    ].
    ^ self
        printOn:aStream 
        format:fmt.

    "
     TimeDuration hours:0 minutes:0 seconds:0 millis:123       
     TimeDuration hours:0 minutes:0 seconds:10 millis:123       
     TimeDuration hours:0 minutes:33 seconds:0 millis:123       
     TimeDuration hours:2 minutes:33 seconds:0 millis:123       
     TimeDuration hours:100 minutes:33 seconds:0 millis:123    
     TimeDuration hours:10000 minutes:33 seconds:0 millis:123    
     TimeDuration hours:1000000 minutes:33 seconds:0 millis:123    

     (TimeDuration hours:0 minutes:0 seconds:0 millis:123) printStringFormat:'%h:%m:%s'       
     (TimeDuration hours:0 minutes:0 seconds:10 millis:123) printStringFormat:'%h:%m:%s'         
     (TimeDuration hours:0 minutes:33 seconds:0 millis:123) printStringFormat:'%h:%m:%s'         
     (TimeDuration hours:2 minutes:33 seconds:0 millis:123) printStringFormat:'%h:%m:%s'         
     (TimeDuration hours:100 minutes:33 seconds:0 millis:123) printStringFormat:'%h:%m:%s'      
     (TimeDuration hours:10000 minutes:33 seconds:0 millis:123) printStringFormat:'%h:%m:%s'      
     (TimeDuration hours:1000000 minutes:33 seconds:0 millis:123) printStringFormat:'%h:%m:%s'      
    "
! !

!TimeDuration methodsFor:'private'!

getMilliseconds
    "return the number of milliseconds since midnight"

    ^ timeEncoding
!

getSeconds
    "return the number of seconds since midnight"

    ^ timeEncoding // 1000
!

setHours:h minutes:m seconds:s millis:millis
    "set my time given individual values"

    self setMilliseconds:((h * 60 * 60 ) + (m * 60) + s) * 1000 + millis.
!

setMilliseconds:millis
    "set my time given milliseconds since midnight"

    timeEncoding := millis
!

setSeconds:secs
    "set my timeduration given seconds.
     Notice that (in contrast to Time), there is no modulu opeartions here.
     Duration can be longer than a day"

    self setMilliseconds:(secs * 1000).
! !

!TimeDuration class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic2/Attic/TimeDuration.st,v 1.6 2004-03-08 15:44:37 cg Exp $'
! !
